@@ -1,28 +1,19 @@
-from nose.tools import assert_equals
+from nose.tools import assert_equals, assert_true, assert_raises
 
-from pretenders.http.client import HttpMock, SubClient
+from pretenders.constants import PRETEND_PORT_RANGE, FOREVER
+from pretenders.exceptions import ConfigurationError
+from pretenders.http.client import HTTPMock
+from pretenders.http.tests.integration import get_fake_client
 
 
-class FakeClient(SubClient):
-
-    def get(self, url, *args, **kwargs):
-        url = "{0}{1}".format(self.url, url)
-        return self.http('GET', url=url, *args, **kwargs)
-
-    def post(self, url, *args, **kwargs):
-        url = "{0}{1}".format(self.url, url)
-        return self.http('POST', url=url, *args, **kwargs)
+http_mock = HTTPMock('localhost', 8000)
+fake_client = get_fake_client(http_mock)
 
 
-http_mock = HttpMock('localhost', 8000)
-fake_client = FakeClient('localhost:8000', '/mock')
-
-
-def add_test_preset(url='/fred/test/one',
-                    method="POST",
-                    body='You tested fred well',
+def add_test_preset(rule='POST /fred/test/one',
+                    body=b'You tested fred well',
                     status=200):
-    http_mock.when(url, method).reply(body, status)
+    http_mock.when(rule).reply(body, status)
 
 
 def test_configure_request_and_check_values():
@@ -41,16 +32,17 @@ def test_configure_request_and_check_values():
 
 
 def test_perform_wrong_method_on_configured_url():
-    "Expect this to fail until we implement method matching in the server."
+    "Test method matching in the server."
     http_mock.reset()
     add_test_preset()
-    response = fake_client.get(url='/fred/test/one')
-    assert_equals(response.status, 405)
+
+    response = fake_client.get(url='/fred/test/another')
+    assert_equals(response.status, 404)
 
     historical_call = http_mock.get_request(0)
 
     assert_equals(historical_call.method, 'GET')
-    assert_equals(historical_call.url, '/fred/test/one')
+    assert_equals(historical_call.url, '/fred/test/another')
     assert_equals(historical_call.body, b'')
 
 
@@ -81,10 +73,10 @@ def test_configure_multiple_rules_independent():
         rather than depending on the order of the call.
     """
     http_mock.reset()
-    http_mock.when('/test_200').reply(status=200)
-    http_mock.when('/test_400').reply(status=400)
-    http_mock.when('/test_500').reply(status=500)
-    http_mock.when('/test_410').reply(status=410)
+    http_mock.when('.* /test_200').reply(status=200)
+    http_mock.when('.* /test_400').reply(status=400)
+    http_mock.when('.* /test_500').reply(status=500)
+    http_mock.when('.* /test_410').reply(status=410)
 
     for url, expected_status in [('/test_200', 200),
                                  ('/test_400', 400),
@@ -102,10 +94,10 @@ def test_configure_multiple_rules_url_match():
     """We get responses based on the url used.
     """
     http_mock.reset()
-    http_mock.when('/test_500').reply(status=500)
-    http_mock.when('/test_410').reply(status=410)
-    http_mock.when('/test_200').reply(status=200)
-    http_mock.when('/test_400').reply(status=400)
+    http_mock.when('.* /test_500').reply(status=500)
+    http_mock.when('.* /test_410').reply(status=410)
+    http_mock.when('.* /test_200').reply(status=200)
+    http_mock.when('.* /test_400').reply(status=400)
 
     for url, expected_status in [('/test_200', 200),
                                  ('/test_400', 400),
@@ -120,20 +112,20 @@ def test_configure_multiple_rules_url_match():
 def test_method_matching():
     "Test that server matches methods correctly."
     http_mock.reset()
-    http_mock.when('/test_get', 'GET').reply('You tested a get', 200)
-    http_mock.when('/test_post', 'POST').reply('You tested a post', 201)
-    http_mock.when('/test_star', '.*').reply('You tested a .*', 202)
-    http_mock.when('/test_star', '.*').reply('You tested a .*', 202)
-    http_mock.when('/test_put_or_post', '(PUT|POST)').reply(
-            'You tested a PUT or a POST',  203)
+    http_mock.when('GET /test_get').reply(b'You tested a get', 200)
+    http_mock.when('POST /test_post').reply(b'You tested a post', 201)
+    http_mock.when('.* /test_star').reply(b'You tested a .*', 202)
+    http_mock.when('.* /test_star').reply(b'You tested a .*', 202)
+    http_mock.when('(PUT|POST) /test_put_or_post').reply(
+            b'You tested a PUT or a POST',  203)
 
     # Only GET works when GET matched
-    assert_equals(405, fake_client.post(url="/test_get").status)
+    assert_equals(404, fake_client.post(url="/test_get").status)
     assert_equals(200, fake_client.get(url="/test_get").status)
     assert_equals(404, fake_client.get(url="/test_get").status)
 
     # Only POST works when POST matched
-    assert_equals(405, fake_client.get(url="/test_post").status)
+    assert_equals(404, fake_client.get(url="/test_post").status)
     assert_equals(201, fake_client.post(url="/test_post").status)
     assert_equals(404, fake_client.post(url="/test_post").status)
 
@@ -143,7 +135,7 @@ def test_method_matching():
     assert_equals(404, fake_client.post(url="/test_star").status)
 
     # PUT or POST work with (PUT|POST) as the method matched
-    assert_equals(405, fake_client.get(url="/test_put_or_post").status)
+    assert_equals(404, fake_client.get(url="/test_put_or_post").status)
     assert_equals(203, fake_client.post(url="/test_put_or_post").status)
     assert_equals(404, fake_client.post(url="/test_put_or_post").status)
 
@@ -151,10 +143,10 @@ def test_method_matching():
 def test_multiple_responses_for_a_url():
     "Test each url pattern can have a multitude of responses"
     http_mock.reset()
-    http_mock.when('/test_url_pattern').reply(status=200)
-    http_mock.when('/test_url_pattern_2').reply(status=201)
-    http_mock.when('/test_url_pattern').reply(status=301)
-    http_mock.when('/test_url_pattern').reply(status=410)
+    http_mock.when('.* /test_url_pattern').reply(status=200)
+    http_mock.when('.* /test_url_pattern_2').reply(status=201)
+    http_mock.when('.* /test_url_pattern').reply(status=301)
+    http_mock.when('.* /test_url_pattern').reply(status=410)
 
     for url, expected_status in [('/test_url_pattern', 200),
                                  ('/test_url_pattern', 301),
@@ -168,7 +160,7 @@ def test_multiple_responses_for_a_url():
 
 def test_regular_expression_matching():
     "Test with regular expression matching"
-    url = r'^/something/([a-zA-Z0-9\-_]*)/?'
+    url = r'^GET /something/([a-zA-Z0-9\-_]*)/?'
     http_mock.reset()
     for i in range(5):
         http_mock.when(url).reply(status=200)
@@ -186,16 +178,62 @@ def test_regular_expression_matching():
 def test_blank_url_matches_anything():
     "A blank url matcher header matches any url"
     http_mock.reset()
-    http_mock.when(url='', method='POST').reply(status=200)
+    http_mock.when('POST').reply(status=200)
     response = fake_client.post(url='/some/strange/12121/string')
     assert_equals(response.status, 200)
+    # Check subsequent call 404s
     response = fake_client.post(url='/some/strange/12121/string')
     assert_equals(response.status, 404)
 
 
+def test_setup_reply_multiple_times():
+    "A times argument can be used to reply multiple times for the same url."
+    http_mock.reset()
+    http_mock.when('POST').reply(status=202, times=3)
+    for expected_status in [202, 202, 202, 404]:
+        response = fake_client.post(url='/some/strange/12121/string')
+        assert_equals(response.status, expected_status)
+
+
+def test_setup_reply_forever():
+    """``FOREVER`` should make the url always respond with the given response.
+
+    Clearly this doesn't test that it will *always* return the response. It
+    tests that it does at least do it for 10 requests though.
+    """
+    http_mock.reset()
+    http_mock.when('POST').reply(status=202, times=FOREVER)
+    for _ in range(10):
+        response = fake_client.post(url='/some/strange/12121/string')
+        assert_equals(response.status, 202)
+
+
+def test_setup_with_zero_or_negative_times():
+    "Test that with zero times we get a 404 for the mock requests"
+    http_mock.reset()
+    assert_raises(ConfigurationError,
+                  http_mock.when('POST').reply, status=202, times=0)
+
+    assert_raises(ConfigurationError,
+                  http_mock.when('POST').reply, status=202, times=-2)
+
+
 def test_missing_method_and_url_matches_anything():
     "Missing matcher headers match anything"
     http_mock.reset()
     http_mock.reply(b'Hello', 323)
     response = fake_client.post(url='/some/strange/12121/string')
     assert_equals(response.status, 323)
+
+
+def test_start_http_pretender():
+    """
+    Test that the http client kicks off an server via a call to the boss.
+
+    TODO: This will need updating when we change the server to return
+    dynamic port configurations for the mock server.
+    """
+    new_mock = HTTPMock('localhost', 8000)
+    assert_true(new_mock.pretend_access_point != "localhost:8000")
+    assert_true(int(new_mock.pretend_access_point.split(':')[1])
+                 in PRETEND_PORT_RANGE)
